package tycho.surefire.problem.chart;

import java.util.Arrays;
import java.util.List;

import org.eclipse.birt.chart.model.attribute.ActionType;
import org.eclipse.birt.chart.model.attribute.AxisType;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.IntersectionType;
import org.eclipse.birt.chart.model.attribute.LineStyle;
import org.eclipse.birt.chart.model.attribute.MarkerType;
import org.eclipse.birt.chart.model.attribute.Orientation;
import org.eclipse.birt.chart.model.attribute.Palette;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.TickStyle;
import org.eclipse.birt.chart.model.attribute.TooltipValue;
import org.eclipse.birt.chart.model.attribute.TriggerCondition;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.attribute.impl.JavaDateFormatSpecifierImpl;
import org.eclipse.birt.chart.model.attribute.impl.PaletteImpl;
import org.eclipse.birt.chart.model.attribute.impl.TooltipValueImpl;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.component.MarkerLine;
import org.eclipse.birt.chart.model.component.Series;
import org.eclipse.birt.chart.model.component.impl.MarkerLineImpl;
import org.eclipse.birt.chart.model.component.impl.SeriesImpl;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.data.impl.ActionImpl;
import org.eclipse.birt.chart.model.data.impl.DateTimeDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataElementImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.SeriesDefinitionImpl;
import org.eclipse.birt.chart.model.data.impl.TriggerImpl;
import org.eclipse.birt.chart.model.impl.ChartWithAxesImpl;
import org.eclipse.birt.chart.model.type.LineSeries;
import org.eclipse.birt.chart.model.type.impl.AreaSeriesImpl;
import org.eclipse.birt.chart.model.type.impl.LineSeriesImpl;
import org.eclipse.core.runtime.Assert;

public class LinesChart
extends ChartWithAxesImpl {

	public static final class LineData {

		private final String			label;
		private final ColorDefinition	color;
		private final List<Double>		values;
		private final boolean			fill;

		public LineData(final String label, final ColorDefinition color, final Double[] values, final boolean fill) {
			Assert.isTrue(label != null && !label.trim().isEmpty());
			Assert.isTrue(values != null && values.length > 0);
			Assert.isNotNull(color);
			this.values = Arrays.asList(values);
			this.label = label;
			this.color = color;
			this.fill = fill;
		}

	}

	private static final String	VERDANA	= "Verdana";	//$NON-NLS-1$

	public LinesChart(final String title, final long[] dates, final List<LineData> lines, final boolean showGrid, final boolean stack) {

		super();

		Assert.isTrue(lines != null && !lines.isEmpty());

		for (final LineData line : lines) {
			Assert.isTrue(dates.length == line.values.size());
		}

		this.initDefault();

		// Legend

		this.getLegend().setOrientation(Orientation.HORIZONTAL_LITERAL);
		this.getLegend().setPosition(Position.BELOW_LITERAL);
		this.getLegend().getText().getFont().setSize(10);

		this.getLegend().getTriggers().add(TriggerImpl.create(TriggerCondition.ONCLICK_LITERAL, ActionImpl.create(ActionType.TOGGLE_VISIBILITY_LITERAL, null)));

		// --------------------- Title ---------------------

		this.getTitle().getLabel().getCaption().setValue(title);
		this.getTitle().getLabel().getCaption().getFont().setSize(10);
		this.getTitle().setVisible(title != null && !title.trim().isEmpty());

		// --------------------- X-things ---------------------

		final Series xSeries = SeriesImpl.create();
		xSeries.setDataSet(DateTimeDataSetImpl.create(dates));

		final SeriesDefinition xSeriesDefinition = SeriesDefinitionImpl.create();
		xSeriesDefinition.getSeries().add(xSeries);

		final Axis xAxis = this.getPrimaryBaseAxes()[0];
		xAxis.setFormatSpecifier(JavaDateFormatSpecifierImpl.create("dd-MM-yy HH:mm")); //$NON-NLS-1$
		xAxis.setType(AxisType.DATE_TIME_LITERAL);
		xAxis.setInterval(3);

		xAxis.getLabel().getCaption().getFont().setName(LinesChart.VERDANA);
		xAxis.getLabel().getCaption().getFont().setSize(10);

		xAxis.getMajorGrid().getLineAttributes().setStyle(LineStyle.DOTTED_LITERAL);
		xAxis.getMajorGrid().getLineAttributes().setColor(ColorDefinitionImpl.GREY());
		xAxis.getMajorGrid().getLineAttributes().setVisible(showGrid);

		xAxis.getMajorGrid().setTickStyle(TickStyle.BELOW_LITERAL);
		xAxis.getOrigin().setType(IntersectionType.MIN_LITERAL);

		xAxis.getSeriesDefinitions().add(xSeriesDefinition);

		// --------------------- Y-things ---------------------

		final Axis yAxis = this.getPrimaryOrthogonalAxis(xAxis);
		yAxis.getLabel().getCaption().getFont().setName(LinesChart.VERDANA);
		yAxis.getLabel().getCaption().getFont().setSize(10);

		yAxis.setType(AxisType.LINEAR_LITERAL);
		yAxis.getOrigin().setType(IntersectionType.VALUE_LITERAL);

		yAxis.getMajorGrid().getLineAttributes().setStyle(LineStyle.DOTTED_LITERAL);
		yAxis.getMajorGrid().getLineAttributes().setColor(ColorDefinitionImpl.GREY());
		yAxis.getMajorGrid().getLineAttributes().setVisible(showGrid);

		// --------------------- Marker Line at y = 0 ---------------------

		final MarkerLine ml = MarkerLineImpl.create(yAxis, NumberDataElementImpl.create(0), ColorDefinitionImpl.BLACK());
		ml.getLineAttributes().setStyle(LineStyle.SOLID_LITERAL);
		ml.getLabel().setVisible(false);

		final SeriesDefinition ySeriesDefinition = SeriesDefinitionImpl.create();
		final Palette seriesPalette = PaletteImpl.create(0, true);

		for (final LineData line : lines) {

			seriesPalette.getEntries().add(line.color.copyInstance());

			final LineSeries series = (LineSeries) (line.fill ? AreaSeriesImpl.create() : LineSeriesImpl.create());
			series.getLineAttributes().setColor(line.color.copyInstance());
			series.getLineAttributes().setThickness(2);
			series.setDataSet(NumberDataSetImpl.create(line.values));
			series.setSeriesIdentifier(line.label);
			series.setPaletteLineColor(false);
			series.setStacked(stack);

			for (int j = 0; j < series.getMarkers().size(); j++) {
				series.getMarkers().get(j).setSize(2);
				series.getMarkers().get(j).setType(MarkerType.CIRCLE_LITERAL);
			}

			/*
			 * TODO temporary commented out for packaging matters
			 * final TooltipValue tooltipValue = new CustomTooltipValueImpl() {
			 * @Override
			 * public String getText(final DataPointHints dph) {
			 * return dph.getSeriesDisplayValue()
			 * + " value was " //$NON-NLS-1$
			 * + dph.getDisplayValue()
			 * + " on " //$NON-NLS-1$
			 * + dph.getBaseDisplayValue();
			 * }
			 * };
			 */

			final TooltipValue tooltipValue = TooltipValueImpl.create(500, null);

			series.getTriggers().add(TriggerImpl.create(TriggerCondition.ONMOUSEOVER_LITERAL, ActionImpl.create(ActionType.SHOW_TOOLTIP_LITERAL, tooltipValue)));
			series.getTriggers().add(TriggerImpl.create(TriggerCondition.ONCLICK_LITERAL, ActionImpl.create(ActionType.TOGGLE_VISIBILITY_LITERAL, null)));

			ySeriesDefinition.getSeries().add(series);
		}

		ySeriesDefinition.setSeriesPalette(seriesPalette);
		yAxis.getSeriesDefinitions().add(ySeriesDefinition);

	}
}
